package projectMonorail.DrawingObjects;

import projectMonorail.DirectionType;
import projectMonorail.Entities.*;
import projectMonorail.Extras.*;
import projectMonorail.MovementStrategy.*;

import java.awt.*;
import java.util.Random;

public class DrawingLocomotive {

    private EntityLocomotive entityLocomotive;

    public EntityLocomotive getEntityLocomotive() {
        return entityLocomotive;
    }

    protected void setEntityLocomotive(EntityLocomotive entityLocomotive) {
        this.entityLocomotive = entityLocomotive;
    }

    private IDrawingWheels drawingWheels;

    private int pictureWidth;

    private int pictureHeight;

    protected int startPosX;

    protected int startPosY;

    protected int locomotiveWidth = 117;

    protected int locomotiveHeight = 56;

    public int getPosX() {
        return startPosX;
    }

    public int getPosY() {
        return startPosY;
    }

    public int getWidth() {
        return locomotiveWidth;
    }

    public int getHeight() {
        return locomotiveHeight;
    }

    public IMoveableObject getMoveableObject() {
        return new DrawingObjectLocomotive(this);
    }

    public DrawingLocomotive(int speed, double weight, Color mainColor, int width, int height, int wheelNumber) {
        if (width < locomotiveWidth || height < locomotiveHeight) {
            return;
        }
        pictureWidth = width;
        pictureHeight = height;
        entityLocomotive = new EntityLocomotive(speed, weight, mainColor);
        Random rand = new Random();
        drawingWheels = switch (rand.nextInt(0, 3)) {
            case 0 -> new DrawingNormalWheels();
            case 1 -> new DrawingSquarePatternWheels();
            case 2 -> new DrawingVintageWheels();
            default -> new DrawingNormalWheels();
        };
        drawingWheels.setWheelNumber(wheelNumber);
    }

    protected DrawingLocomotive(int speed, double weight, Color mainColor,
                                int width, int height, int locoWidth, int locoHeight, int wheelNumber) {
        if (width < locoWidth || height < locoHeight) {
            return;
        }
        pictureWidth = width;
        pictureHeight = height;
        locomotiveWidth = locoWidth;
        locomotiveHeight = locoHeight;
        entityLocomotive = new EntityLocomotive(speed, weight, mainColor);
        Random rand = new Random();
        drawingWheels = switch (rand.nextInt(0, 3)) {
            case 0 -> new DrawingNormalWheels();
            case 1 -> new DrawingSquarePatternWheels();
            case 2 -> new DrawingVintageWheels();
            default -> new DrawingNormalWheels();
        };
        drawingWheels.setWheelNumber(wheelNumber);
    }

    public DrawingLocomotive(EntityLocomotive entityLocomotive, int width, int height, int wheelNumber,
                             IDrawingWheels drawingWheels) {
        if (width < locomotiveWidth || height < locomotiveHeight) {
            return;
        }
        pictureWidth = width;
        pictureHeight = height;
        this.entityLocomotive = entityLocomotive;
        this.drawingWheels = drawingWheels;
        drawingWheels.setWheelNumber(wheelNumber);
    }

    public void setPosition(int x, int y) {
        if (x < 0 || x + locomotiveWidth > pictureWidth) {
            x = 0;
        }
        if (y < 0 || y + locomotiveHeight > pictureHeight) {
            y = 0;
        }

        startPosX = x;
        startPosY = y;
    }

    public boolean canMove(DirectionType direction) {
        if (entityLocomotive == null) {
            return false;
        }

        return switch (direction) {
            case Left -> startPosX - entityLocomotive.getStep() > 0;
            case Up -> startPosY - entityLocomotive.getStep() > 0;
            case Right -> startPosX + locomotiveWidth + entityLocomotive.getStep() < pictureWidth;
            case Down -> startPosY + locomotiveHeight + entityLocomotive.getStep() < pictureHeight;
            default -> false;
        };
    }

    public void moveTransport(DirectionType direction) {
        if (!canMove(direction) || entityLocomotive == null) {
            return;
        }

        switch (direction) {
            case Left:
                startPosX -= (int) entityLocomotive.getStep();
                break;

            case Up:
                startPosY -= (int) entityLocomotive.getStep();
                break;

            case Right:
                startPosX += (int) entityLocomotive.getStep();
                break;

            case Down:
                startPosY += (int) entityLocomotive.getStep();
                break;
        }
    }

    public void drawTransport(Graphics2D g2d) {
        if (entityLocomotive == null) {
            return;
        }

        Color mainColor = entityLocomotive.getMainColor();
        BasicStroke standardWidth = new BasicStroke(1);
        BasicStroke largerWidth = new BasicStroke(2);

        //надстройка
        g2d.setStroke(largerWidth);
        g2d.setColor(mainColor);
        g2d.fillRect(startPosX + 55, startPosY, 25, 15);

        g2d.setColor(Color.BLACK);
        g2d.drawRect(startPosX + 55, startPosY, 25, 15);

        //корпус локомотива
        Polygon locoPolygon = new Polygon();
        locoPolygon.addPoint(startPosX + 29, startPosY + 15);
        locoPolygon.addPoint(startPosX + 112, startPosY + 15);
        locoPolygon.addPoint(startPosX + 112, startPosY + 46);
        locoPolygon.addPoint(startPosX + 25, startPosY + 46);
        locoPolygon.addPoint(startPosX + 25, startPosY + 31);

        g2d.setColor(mainColor);
        g2d.fillPolygon(locoPolygon);

        g2d.setColor(Color.BLACK);
        g2d.drawPolygon(locoPolygon);

        g2d.setStroke(standardWidth);
        g2d.setColor(Color.BLUE);
        g2d.drawLine(startPosX + 25, startPosY + 31, startPosX + 112, startPosY + 31);

        //дверь локомотива
        g2d.setColor(Color.GRAY);
        g2d.fillRect(startPosX + 54, startPosY + 21, 7, 20);

        g2d.setStroke(largerWidth);
        g2d.setColor(Color.BLACK);
        g2d.drawRect(startPosX + 54, startPosY + 21, 7, 20);

        //окна локомотива
        g2d.setColor(Color.BLUE);

        g2d.fillRect(startPosX + 32, startPosY + 18, 6, 9);
        g2d.fillRect(startPosX + 44, startPosY + 18, 6, 9);
        g2d.fillRect(startPosX + 103, startPosY + 18, 6, 9);

        g2d.setColor(Color.BLACK);

        g2d.drawRect(startPosX + 32, startPosY + 18, 6, 9);
        g2d.drawRect(startPosX + 44, startPosY + 18, 6, 9);
        g2d.drawRect(startPosX + 103, startPosY + 18, 6, 9);

        //колеса и тележка локомотива
        g2d.fillRect(startPosX + 23, startPosY + 47, 33, 6);
        g2d.fillRect(startPosX + 76, startPosY + 47, 30, 6);
        g2d.drawRect(startPosX + 23, startPosY + 47, 33, 6);
        g2d.drawRect(startPosX + 76, startPosY + 47, 30, 6);

        drawingWheels.drawWheels(g2d, Color.WHITE, startPosX, startPosY);

        Polygon bogiePolygon = new Polygon();
        bogiePolygon.addPoint(startPosX + 26, startPosY + 46);
        bogiePolygon.addPoint(startPosX + 24, startPosY + 54);
        bogiePolygon.addPoint(startPosX + 12, startPosY + 54);
        bogiePolygon.addPoint(startPosX + 8, startPosY + 51);
        bogiePolygon.addPoint(startPosX + 12, startPosY + 48);
        bogiePolygon.addPoint(startPosX + 18, startPosY + 46);

        g2d.fillPolygon(bogiePolygon);

        //соединение между кабинами
        g2d.drawRect(startPosX + 112, startPosY + 18, 5, 28);
        g2d.fillRect(startPosX + 112, startPosY + 18, 5, 28);
    }
}
