package projectMonorail.DrawingObjects;

import projectMonorail.Entities.*;
import projectMonorail.Extras.*;

import java.awt.*;

public class DrawingMonorail extends DrawingLocomotive {

    public DrawingMonorail(int speed, double weight, Color mainColor, Color additionalColor,
                           boolean magneticRail, boolean extraCabin, int width, int height, int wheelNumber) {
        super(speed, weight, mainColor, width, height, 186, 92, wheelNumber);

        if (!magneticRail && !extraCabin) {
            locomotiveWidth = 117;
            locomotiveHeight = 56;
        }
        if (!magneticRail && extraCabin) {
            locomotiveWidth = 183;
            locomotiveHeight = 56;
        }

        if (getEntityLocomotive() != null) {
            setEntityLocomotive(new EntityMonorail(speed, weight, mainColor,
                    additionalColor, magneticRail, extraCabin));
        }
    }

    public DrawingMonorail(EntityMonorail entityMonorail, int width, int height, int wheelNumber,
                           IDrawingWheels drawingWheels) {
        super(entityMonorail, width, height, wheelNumber, drawingWheels);
    }

    @Override
    public void drawTransport(Graphics2D g2d) {
        if (!(getEntityLocomotive() instanceof EntityMonorail monorail)) {
            return;
        }

        super.drawTransport(g2d);

        Color mainColor = monorail.getMainColor();
        Color additionalColor = monorail.getAdditionalColor();
        BasicStroke standardWidth = new BasicStroke(1);
        BasicStroke largerWidth = new BasicStroke(2);

        //магнитная рельса
        if (monorail.getMagneticRail()) {
            g2d.drawRect(startPosX + 2, startPosY + 58, 184, 18);
            g2d.setColor(Color.GRAY);
            g2d.fillRect(startPosX + 2, startPosY + 58, 184, 18);
            for (int i = 0; i < 4; i++) {
                g2d.setColor(Color.BLACK);
                g2d.drawRect(startPosX + 35 + 35 * i, startPosY + 77, 8, 15);
                g2d.setColor(Color.GRAY);
                g2d.fillRect(startPosX + 35 + 35 * i, startPosY + 77, 8, 15);
            }
        }

        //дополнительная кабина
        if (monorail.getExtraCabin()) {
            //корпус дополнительной кабины
            g2d.setColor(additionalColor);
            g2d.fillRect(startPosX + 118, startPosY + 15, 65, 31);
            g2d.setColor(Color.BLACK);
            g2d.drawRect(startPosX + 118, startPosY + 15, 65, 31);

            g2d.setStroke(standardWidth);
            g2d.setColor(Color.BLUE);
            g2d.drawLine(startPosX + 118, startPosY + 31, startPosX + 183, startPosY + 31);

            //дверь дополнительной кабины
            g2d.setColor(Color.GRAY);
            g2d.fillRect(startPosX + 146, startPosY + 21, 7, 20);

            g2d.setStroke(largerWidth);
            g2d.setColor(Color.BLACK);
            g2d.drawRect(startPosX + 146, startPosY + 21, 7, 20);

            //окна дополнительной кабины
            g2d.setColor(Color.BLUE);
            g2d.fillRect(startPosX + 130, startPosY + 18, 6, 9);
            g2d.fillRect(startPosX + 169, startPosY + 18, 6, 9);

            g2d.setColor(Color.BLACK);
            g2d.drawRect(startPosX + 130, startPosY + 18, 6, 9);
            g2d.drawRect(startPosX + 169, startPosY + 18, 6, 9);

            //колеса и тележка дополнительной кабины
            g2d.fillRect(startPosX + 126, startPosY + 47, 15, 6);
            g2d.fillRect(startPosX + 159, startPosY + 47, 15, 6);
            g2d.drawRect(startPosX + 126, startPosY + 47, 15, 6);
            g2d.drawRect(startPosX + 159, startPosY + 47, 15, 6);

            g2d.setColor(Color.WHITE);
            g2d.fillOval(startPosX + 128, startPosY + 47, 10, 9);
            g2d.fillOval(startPosX + 161, startPosY + 47, 10, 9);

            g2d.setColor(Color.BLACK);
            g2d.drawOval(startPosX + 128, startPosY + 47, 10, 9);
            g2d.drawOval(startPosX + 161, startPosY + 47, 10, 9);
        }
    }

}
