package projectMonorail.Extras;

import projectMonorail.DrawingObjects.DrawingLocomotive;
import projectMonorail.DrawingObjects.DrawingMonorail;
import projectMonorail.Entities.*;

import java.util.concurrent.ThreadLocalRandom;

public class RandomGeneric<E extends EntityLocomotive, D extends IDrawingWheels> {
    private final E[] locomotives;

    private int locomotivesCurrNum = 0;

    private final D[] wheels;

    private int wheelsCurrNum = 0;

    private int pictureWidth;

    private int pictureHeight;

    public RandomGeneric(int locomotivesCount, int drawingsCount, int pictureWidth, int pictureHeight) {
        locomotives = (E[])new EntityLocomotive[locomotivesCount];
        wheels = (D[])new IDrawingWheels[drawingsCount];
        this.pictureWidth = pictureWidth;
        this.pictureHeight = pictureHeight;
    }

    public void addItem(E locomotive) {
        if (locomotive == null || locomotivesCurrNum >= locomotives.length)
            return;

        locomotives[locomotivesCurrNum++] = locomotive;
    }

    public void addItem(D drawingWheels) {
        if (drawingWheels == null || wheelsCurrNum >= wheels.length)
            return;

        wheels[wheelsCurrNum++] = drawingWheels;
    }

    public DrawingLocomotive getDrawingLocomotive() {
        if (locomotivesCurrNum == 0 || wheelsCurrNum == 0)
            return null;

        EntityLocomotive locomotive = locomotives[ThreadLocalRandom.current().nextInt(0, locomotivesCurrNum)];
        IDrawingWheels drawingWheels = wheels[ThreadLocalRandom.current().nextInt(0, wheelsCurrNum)];

        if (locomotive instanceof EntityMonorail) {
            return new DrawingMonorail((EntityMonorail) locomotive, pictureWidth, pictureHeight, ThreadLocalRandom.current().nextInt(2, 5),
                    drawingWheels);
        }

        return new DrawingLocomotive(locomotive, pictureWidth, pictureHeight, ThreadLocalRandom.current().nextInt(2, 5),
                drawingWheels);
    }
}
