package projectMonorail;

import projectMonorail.DrawingObjects.*;
import projectMonorail.Generics.*;
import projectMonorail.MovementStrategy.*;

import javax.swing.*;
import javax.swing.text.NumberFormatter;
import java.awt.*;
import java.text.NumberFormat;

public class FrameMonorailCollection extends JFrame {

    private final LocomotivesGenericCollection<DrawingLocomotive, DrawingObjectLocomotive> locomotives;

    private JPanel paddingTopPanel;

    private JPanel mainPanel;

    private PictureBoxCollection pictureBoxCollection;

    private int pictureBoxCollectionWidth = 900;

    private int pictureBoxCollectionHeight = 430;

    private JPanel toolsPanel;

    private JLabel nameLabel;

    private JButton buttonAddMonorail;

    private JButton buttonRemoveMonorail;

    private JButton buttonRefreshCollection;

    private JButton buttonRandomGeneration;

    private JFormattedTextField textFieldNumber;

    public FrameMonorailCollection() {
        super("Monorail collection");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        locomotives = new LocomotivesGenericCollection<>(pictureBoxCollectionWidth, pictureBoxCollectionHeight);
        pictureBoxCollection = new PictureBoxCollection(locomotives, pictureBoxCollectionWidth,
                pictureBoxCollectionHeight);

        nameLabel = new JLabel("    Tools");
        nameLabel.setFont(new Font("Segoe UI", Font.BOLD, 16));
        nameLabel.setHorizontalTextPosition(JLabel.LEFT);
        nameLabel.setMaximumSize(new Dimension(190, 30));
        nameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonAddMonorail = new JButton("Add monorail");
        buttonAddMonorail.setHorizontalTextPosition(JButton.CENTER);
        buttonAddMonorail.setVerticalTextPosition(JButton.CENTER);
        buttonAddMonorail.setFocusable(false);
        buttonAddMonorail.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonAddMonorail.setBackground(Color.WHITE);
        buttonAddMonorail.setMaximumSize(new Dimension(155, 36));
        buttonAddMonorail.setBorder(BorderFactory.createLineBorder(Color.black, 2));
        buttonAddMonorail.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonAddMonorail.addActionListener(e -> {
            FrameMonorail frame = new FrameMonorail(this);
            if (frame.getPictureBox().getDialogResult()) {
                if (locomotives.addition(frame.getPictureBox().getSelectedLocomotive()) != -1) {
                    JOptionPane.showMessageDialog(null, "Object added", "Message",
                            JOptionPane.INFORMATION_MESSAGE);
                    pictureBoxCollection.repaint();
                } else {
                    JOptionPane.showMessageDialog(null, "Failed to add the object", "Message",
                            JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });

        NumberFormat format = NumberFormat.getInstance();
        NumberFormatter formatter = new NumberFormatter(format);
        formatter.setValueClass(Integer.class);
        formatter.setMinimum(0);
        formatter.setMaximum(Integer.MAX_VALUE);
        formatter.setAllowsInvalid(false);
        formatter.setCommitsOnValidEdit(true);

        textFieldNumber = new JFormattedTextField(formatter);
        textFieldNumber.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        textFieldNumber.setMaximumSize(new Dimension(155, 25));
        textFieldNumber.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonRemoveMonorail = new JButton("Remove monorail");
        buttonRemoveMonorail.setHorizontalTextPosition(JButton.CENTER);
        buttonRemoveMonorail.setVerticalTextPosition(JButton.CENTER);
        buttonRemoveMonorail.setFocusable(false);
        buttonRemoveMonorail.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonRemoveMonorail.setBackground(Color.WHITE);
        buttonRemoveMonorail.setMaximumSize(new Dimension(155, 36));
        buttonRemoveMonorail.setBorder(BorderFactory.createLineBorder(Color.black, 2));
        buttonRemoveMonorail.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonRemoveMonorail.addActionListener(e -> {
            if (JOptionPane.showConfirmDialog(null, "Delete the object?", "Deletion",
                    JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
                return;
            }
            if (textFieldNumber.getText().isEmpty()) {
                return;
            }
            int pos = Integer.parseInt(textFieldNumber.getText());
            if (locomotives.subtraction(pos)) {
                JOptionPane.showMessageDialog(null, "Object deleted", "Message",
                        JOptionPane.INFORMATION_MESSAGE);
                pictureBoxCollection.repaint();
            } else {
                JOptionPane.showMessageDialog(null, "Failed to delete the object", "Message",
                        JOptionPane.INFORMATION_MESSAGE);
            }
        });

        buttonRefreshCollection = new JButton("Refresh collection");
        buttonRefreshCollection.setHorizontalTextPosition(JButton.CENTER);
        buttonRefreshCollection.setVerticalTextPosition(JButton.CENTER);
        buttonRefreshCollection.setFocusable(false);
        buttonRefreshCollection.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonRefreshCollection.setBackground(Color.WHITE);
        buttonRefreshCollection.setMaximumSize(new Dimension(155, 36));
        buttonRefreshCollection.setBorder(BorderFactory.createLineBorder(Color.black, 2));
        buttonRefreshCollection.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonRefreshCollection.addActionListener(e -> pictureBoxCollection.repaint());

        buttonRandomGeneration = new JButton("Show additional frame");
        buttonRandomGeneration.setHorizontalTextPosition(JButton.CENTER);
        buttonRandomGeneration.setVerticalTextPosition(JButton.CENTER);
        buttonRandomGeneration.setFocusable(false);
        buttonRandomGeneration.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonRandomGeneration.setBackground(Color.WHITE);
        buttonRandomGeneration.setMaximumSize(new Dimension(155, 36));
        buttonRandomGeneration.setBorder(BorderFactory.createLineBorder(Color.black, 2));
        buttonRandomGeneration.setAlignmentX(Component.CENTER_ALIGNMENT);

        buttonRandomGeneration.addActionListener(e -> new FrameRandomGeneration(this));

        toolsPanel = new JPanel();
        toolsPanel.setMaximumSize(new Dimension(190, 430));
        toolsPanel.setLayout(new BoxLayout(toolsPanel, BoxLayout.Y_AXIS));
        toolsPanel.setBorder(BorderFactory.createLineBorder(Color.black));
        toolsPanel.add(nameLabel);
        toolsPanel.add(Box.createVerticalStrut(14));
        toolsPanel.add(buttonAddMonorail);
        toolsPanel.add(Box.createVerticalStrut(77));
        toolsPanel.add(textFieldNumber);
        toolsPanel.add(Box.createVerticalStrut(6));
        toolsPanel.add(buttonRemoveMonorail);
        toolsPanel.add(Box.createVerticalStrut(74));
        toolsPanel.add(buttonRefreshCollection);
        toolsPanel.add(Box.createVerticalStrut(40));
        toolsPanel.add(buttonRandomGeneration);

        mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
        mainPanel.add(Box.createHorizontalStrut(10));
        mainPanel.add(pictureBoxCollection);
        mainPanel.add(Box.createHorizontalStrut(15));
        mainPanel.add(toolsPanel);

        paddingTopPanel = new JPanel();
        paddingTopPanel.setLayout(new BoxLayout(paddingTopPanel, BoxLayout.Y_AXIS));
        paddingTopPanel.add(Box.createVerticalStrut(15));
        paddingTopPanel.add(mainPanel);

        add(paddingTopPanel);

        setPreferredSize(new Dimension(990, 478));
        pack();
        setLocationRelativeTo(null);
        setVisible(true);
    }
}
