package projectMonorail.Generics;
import projectMonorail.DrawingObjects.*;
import projectMonorail.MovementStrategy.*;

import java.awt.*;

public class LocomotivesGenericCollection<T extends DrawingLocomotive, U extends IMoveableObject> {
    private final int pictureWidth;

    private final int pictureHeight;

    private final int placeSizeWidth = 193;

    private final int placeSizeHeight = 102;

    private final SetGeneric<T> collection;

    public LocomotivesGenericCollection(int picWidth, int picHeight) {
        int width = picWidth / placeSizeWidth;
        int height = picHeight / placeSizeHeight;
        pictureWidth = picWidth;
        pictureHeight = picHeight;
        collection = new SetGeneric<T>(width * height);
    }

    public int addition(T obj) {
        if (obj == null)
        {
            return -1;
        }
        return collection.insert(obj);
    }

    public boolean subtraction(int pos) {
        T obj = collection.get(pos);
        if (obj != null)
        {
            return collection.remove(pos);
        }
        return false;
    }

    public U getU(int pos) {
        if (collection.get(pos) != null)
            return (U)collection.get(pos).getMoveableObject();
        return null;
    }

    public void showLocomotives(Graphics2D g2d) {
        DrawBackground(g2d);
        DrawObjects(g2d);
    }

    private void DrawBackground(Graphics2D g2d) {
        g2d.setColor(Color.BLACK);
        g2d.setStroke(new BasicStroke(3));
        for (int i = 0; i < pictureWidth / placeSizeWidth; i++) {
            for (int j = 0; j < pictureHeight / placeSizeHeight + 1; ++j) {
                g2d.drawLine(i * placeSizeWidth, j * placeSizeHeight, i * placeSizeWidth
                        + placeSizeWidth / 2, j * placeSizeHeight);
            }
            g2d.drawLine( i * placeSizeWidth, 0, i * placeSizeWidth,
                    pictureHeight / placeSizeHeight * placeSizeHeight);
        }
    }

    private void DrawObjects(Graphics2D g2d) {
        T obj;
        int width = pictureWidth / placeSizeWidth;
        int height = pictureHeight / placeSizeHeight;
        int diff = 1, currWidth = 0;
        for (int i = 0; i < collection.count(); i++) {
            currWidth++;
            if (currWidth > width) {
                diff++;
                currWidth = 1;
            }
            obj = collection.get(i);
            if (obj != null) {
                obj.setPosition(i % width * placeSizeWidth + placeSizeWidth / 40,
                        (height - diff) * placeSizeHeight + placeSizeHeight / 15);
                obj.drawTransport(g2d);
            }
        }
    }

}
