package projectMonorail;

import projectMonorail.DrawingObjects.*;
import projectMonorail.MovementStrategy.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.util.Random;

public class PictureBox extends JPanel {
    private boolean dialogResult = false;

    public boolean getDialogResult() {
        return dialogResult;
    }

    private DrawingLocomotive drawingLocomotive;

    private AbstractStrategy abstractStrategy;

    private DrawingLocomotive selectedLocomotive;

    public DrawingLocomotive getSelectedLocomotive() {
        return selectedLocomotive;
    }

    private JButton buttonLeft;

    private JButton buttonUp;

    private JButton buttonRight;

    private JButton buttonDown;

    private JButton buttonCreateLocomotive;

    private JButton buttonCreateMonorail;

    private JButton buttonSelectLocomotive;

    private JButton buttonStep;

    private JComboBox comboBoxStrategy;

    private JPanel buttonsPanel;

    private JPanel buttonsMovePanel;

    private JPanel movePaddingPanel;

    private JPanel strategyPaddingPanel;

    private JPanel strategyPanel;

    public PictureBox() {
        abstractStrategy = null;
        selectedLocomotive = null;

        setLayout(new BorderLayout());

        buttonsPanel = new JPanel();
        buttonsPanel.setLayout(new BoxLayout(buttonsPanel, BoxLayout.X_AXIS));
        buttonsPanel.setOpaque(false);

        buttonCreateLocomotive = new JButton("Create locomotive");
        buttonCreateLocomotive.setFocusable(false);
        buttonCreateLocomotive.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonCreateLocomotive.setBackground(Color.LIGHT_GRAY);
        buttonCreateLocomotive.setMaximumSize(new Dimension(140, 42));

        buttonCreateLocomotive.addActionListener(e -> {
            Random random = new Random();
            Color color = new Color(random.nextInt(0, 256), random.nextInt(0, 256), random.nextInt(0, 256));
            Color chosenColor = JColorChooser.showDialog(this, "Main color", Color.WHITE);
            if (chosenColor != null) {
                color = chosenColor;
            }
            drawingLocomotive = new DrawingLocomotive(random.nextInt(200, 300),
                    random.nextInt(1000, 3000),
                    color, this.getWidth(), this.getHeight(), random.nextInt(2, 5));
            drawingLocomotive.setPosition(random.nextInt(10, 100), random.nextInt(10, 100));
            repaint();
        });

        buttonCreateMonorail = new JButton("Create monorail");
        buttonCreateMonorail.setFocusable(false);
        buttonCreateMonorail.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonCreateMonorail.setBackground(Color.LIGHT_GRAY);
        buttonCreateMonorail.setMaximumSize(new Dimension(140, 42));

        buttonCreateMonorail.addActionListener(e -> {
            Random random = new Random();
            Color color = new Color(random.nextInt(0, 256), random.nextInt(0, 256), random.nextInt(0, 256));
            Color chosenColor = JColorChooser.showDialog(this, "Main color", Color.WHITE);
            if (chosenColor != null) {
                color = chosenColor;
            }
            Color additionalColor = new Color(random.nextInt(0, 256), random.nextInt(0, 256), random.nextInt(0, 256));
            Color chosenAddColor = JColorChooser.showDialog(this, "Additional Color", Color.WHITE);
            if (chosenAddColor != null) {
                additionalColor = chosenAddColor;
            }
            drawingLocomotive = new DrawingMonorail(random.nextInt(200, 300),
                    random.nextInt(1000, 3000),
                    color, additionalColor, random.nextBoolean(), random.nextBoolean(),
                    this.getWidth(), this.getHeight(), random.nextInt(2, 5));
            drawingLocomotive.setPosition(random.nextInt(10, 100), random.nextInt(10, 100));
            repaint();
        });

        buttonSelectLocomotive = new JButton("Select");
        buttonSelectLocomotive.setFocusable(false);
        buttonSelectLocomotive.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonSelectLocomotive.setBackground(Color.LIGHT_GRAY);
        buttonSelectLocomotive.setMaximumSize(new Dimension(140, 42));

        buttonSelectLocomotive.addActionListener(e -> {
            selectedLocomotive = drawingLocomotive;
            dialogResult = true;
            JComponent comp = (JComponent)e.getSource();
            Window win = SwingUtilities.getWindowAncestor(comp);
            win.dispose();
        });

        ActionListener buttonMoveListener = e -> {
            if (drawingLocomotive == null) {
                return;
            }

            String buttonName = ((JButton) e.getSource()).getName();

            switch (buttonName) {
                case ("buttonUp"):
                    drawingLocomotive.moveTransport(DirectionType.Up);
                    break;
                case ("buttonDown"):
                    drawingLocomotive.moveTransport(DirectionType.Down);
                    break;
                case ("buttonLeft"):
                    drawingLocomotive.moveTransport(DirectionType.Left);
                    break;
                case ("buttonRight"):
                    drawingLocomotive.moveTransport(DirectionType.Right);
                    break;
            }
            repaint();
        };

        buttonLeft = new JButton();
        buttonLeft.setName("buttonLeft");
        buttonLeft.setFocusable(false);
        buttonLeft.setPreferredSize(new Dimension(30, 30));
        buttonLeft.setIcon(new ImageIcon("Resources/arrowLeft.png"));
        buttonLeft.addActionListener(buttonMoveListener);

        buttonRight = new JButton();
        buttonRight.setName("buttonRight");
        buttonRight.setFocusable(false);
        buttonRight.setPreferredSize(new Dimension(30, 30));
        buttonRight.setIcon(new ImageIcon("Resources/arrowRight.png"));
        buttonRight.addActionListener(buttonMoveListener);

        buttonDown = new JButton();
        buttonDown.setName("buttonDown");
        buttonDown.setFocusable(false);
        buttonDown.setPreferredSize(new Dimension(30, 30));
        buttonDown.setIcon(new ImageIcon("Resources/arrowDown.png"));
        buttonDown.addActionListener(buttonMoveListener);

        buttonUp = new JButton();
        buttonUp.setName("buttonUp");
        buttonUp.setFocusable(false);
        buttonUp.setPreferredSize(new Dimension(30, 30));
        buttonUp.setIcon(new ImageIcon("Resources/arrowUp.png"));
        buttonUp.addActionListener(buttonMoveListener);

        buttonsMovePanel = new JPanel();
        buttonsMovePanel.setLayout(new GridBagLayout());
        buttonsMovePanel.setOpaque(false);

        GridBagConstraints constrains = new GridBagConstraints();
        constrains.insets = new Insets(5, 5, 5, 5);
        constrains.gridx = 0;
        constrains.gridy = 0;

        buttonsMovePanel.add(Box.createHorizontalStrut(30), constrains);

        constrains.gridx = 1;
        constrains.gridy = 0;

        buttonsMovePanel.add(buttonUp, constrains);

        constrains.gridx = 2;
        constrains.gridy = 0;

        buttonsMovePanel.add(Box.createHorizontalStrut(30), constrains);

        constrains.gridx = 0;
        constrains.gridy = 1;

        buttonsMovePanel.add(buttonLeft, constrains);

        constrains.gridx = 1;
        constrains.gridy = 1;

        buttonsMovePanel.add(buttonDown, constrains);

        constrains.gridx = 2;
        constrains.gridy = 1;

        buttonsMovePanel.add(buttonRight, constrains);

        movePaddingPanel = new JPanel();
        movePaddingPanel.setLayout(new BoxLayout(movePaddingPanel, BoxLayout.Y_AXIS));
        movePaddingPanel.setOpaque(false);

        movePaddingPanel.add(buttonsMovePanel);
        movePaddingPanel.add(Box.createVerticalStrut(22));

        buttonsPanel.add(Box.createHorizontalStrut(12));
        buttonsPanel.add(buttonCreateMonorail);
        buttonsPanel.add(Box.createHorizontalStrut(16));
        buttonsPanel.add(buttonCreateLocomotive);
        buttonsPanel.add(Box.createHorizontalStrut(16));
        buttonsPanel.add(buttonSelectLocomotive);
        buttonsPanel.add(Box.createHorizontalStrut(400));
        buttonsPanel.add(movePaddingPanel);

        add(buttonsPanel, BorderLayout.SOUTH);

        strategyPaddingPanel = new JPanel();
        strategyPaddingPanel.setLayout(new BoxLayout(strategyPaddingPanel, BoxLayout.X_AXIS));
        strategyPaddingPanel.setOpaque(false);

        String[] items = {
                "Form center",
                "Form border"
        };
        comboBoxStrategy = new JComboBox(items);
        comboBoxStrategy.setMaximumSize(new Dimension(150, 23));

        buttonStep = new JButton("Step");
        buttonStep.setFocusable(false);
        buttonStep.setFont(new Font("Segoe UI", Font.PLAIN, 12));
        buttonStep.setBackground(Color.LIGHT_GRAY);
        buttonStep.setMaximumSize(new Dimension(75, 28));

        buttonStep.addActionListener(e -> {
            if (drawingLocomotive == null) {
                return;
            }
            if (comboBoxStrategy.isEnabled()) {
                abstractStrategy = switch (comboBoxStrategy.getSelectedIndex()) {
                    case 0 -> new MoveToCenter();
                    case 1 -> new MoveToBorder();
                    default -> null;
                };
                if (abstractStrategy == null)
                {
                    return;
                }
                abstractStrategy.setData(new DrawingObjectLocomotive(drawingLocomotive), this.getWidth(), this.getHeight());
            }
            if (abstractStrategy == null)
            {
                return;
            }
            comboBoxStrategy.setEnabled(false);
            abstractStrategy.makeStep();
            repaint();
            if (abstractStrategy.getStatus() == Status.Finish)
            {
                comboBoxStrategy.setEnabled(true);
                abstractStrategy = null;
            }
        });

        strategyPanel = new JPanel();
        strategyPanel.setLayout(new BoxLayout(strategyPanel, BoxLayout.Y_AXIS));
        strategyPanel.setOpaque(false);

        strategyPanel.add(Box.createVerticalStrut(10));
        strategyPanel.add(comboBoxStrategy);
        strategyPanel.add(Box.createVerticalStrut(15));
        strategyPanel.add(buttonStep);

        strategyPaddingPanel.add(Box.createHorizontalStrut(735));
        strategyPaddingPanel.add(strategyPanel);

        add(strategyPaddingPanel, BorderLayout.NORTH);

        setPreferredSize(new Dimension(900, 500));
    }

    @Override
    protected void paintComponent(Graphics g) {
        if (drawingLocomotive == null) {
            return;
        }

        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D) g;
        drawingLocomotive.drawTransport(g2d);
    }
}
